@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, DeriveDataTypeable, FlexibleContexts, FlexibleInstances, FunctionalDependencies, MultiParamTypeClasses, ScopedTypeVariables, TemplateHaskell #-}
+{-# LANGUAGE CPP, DeriveDataTypeable, FlexibleContexts, FlexibleInstances, FunctionalDependencies, MultiParamTypeClasses, RecordWildCards, ScopedTypeVariables, TemplateHaskell #-}
 {-# OPTIONS_GHC -Wall #-}
 -- | Access to things that Debian policy says should be in a valid
 -- control file.  The pure functions will not throw ControlFileError
@@ -29,16 +29,18 @@ module Debian.Control.Policy
 
 import Control.Exception (Exception, throw)
 import Control.Monad.Catch (MonadCatch, try)
+import Data.List (intercalate)
 import Data.Text (Text)
 import Data.Typeable (Typeable)
 import Data.ListLike (toList)
 import Debian.Control.Common (Control'(..), Paragraph'(..), Field'(..), fieldValue, ControlFunctions(parseControlFromFile, parseControl))
 import Debian.Control.Text ()
 import Debian.Loc (__LOC__)
-import Debian.Pretty (ppDisplay)
+import Debian.Pretty (prettyShow)
 import Debian.Relation (SrcPkgName(..), BinPkgName(..), Relations, parseRelations)
 import Debian.Relation.Text ()
-import Language.Haskell.TH (Loc)
+import Language.Haskell.TH (Loc(..))
+import Prelude hiding (ioError)
 -- import qualified Debug.ShowPlease as Please
 import Text.Parsec.Error (ParseError)
 
@@ -47,7 +49,7 @@ import Text.Parsec.Error (ParseError)
 data DebianControl = DebianControl {unDebianControl :: Control' Text}
 
 instance Show DebianControl where
-    show c = "(parseDebianControl \"\" " ++ show (ppDisplay (unDebianControl c)) ++ ")"
+    show c = "(parseDebianControl \"\" " ++ show (prettyShow (unDebianControl c)) ++ ")"
 
 -- | Validate and return a control file in an opaque wrapper.  May
 -- throw a ControlFileError.  Currently we only verify that it has a
@@ -75,7 +77,7 @@ parseDebianControlFromFile controlPath =
          (either (return . Left . ParseControlError [$__LOC__]) validateDebianControl)
 
 -- | Class of things that contain a validated Debian control file.
-class HasDebianControl a where
+class Show a => HasDebianControl a where
     debianControl :: a -> DebianControl
 
 instance HasDebianControl DebianControl where
@@ -94,12 +96,23 @@ instance HasControl DebianControl where
 -- line number generated by template haskell.
 data ControlFileError
     = NoParagraphs        {locs :: [Loc]}
-    | NoBinaryParagraphs  {locs :: [Loc]}
+    | NoBinaryParagraphs  {locs :: [Loc], ctl :: String}
     | MissingField        {locs :: [Loc], field :: String}
     | ParseRelationsError {locs :: [Loc], parseError :: ParseError}
     | ParseControlError   {locs :: [Loc], parseError :: ParseError}
     | IOError             {locs :: [Loc], ioError :: IOError}
-    deriving (Show, Typeable)
+    deriving Typeable
+
+instance Show ControlFileError where
+    show (NoParagraphs {..}) = intercalate ", " (map showLoc locs) ++ ": NoParagraphs"
+    show (NoBinaryParagraphs {..}) = intercalate ", " (map showLoc locs) ++ ": NoBinaryParagraphs (" ++ show ctl ++ ")"
+    show (MissingField {..}) = intercalate ", " (map showLoc locs) ++ ": MissingField " ++ show field
+    show (ParseRelationsError {..}) = intercalate ", " (map showLoc locs) ++ ": ParseRelationsError " ++ show parseError
+    show (ParseControlError {..}) = intercalate ", " (map showLoc locs) ++ ": ParseControlError " ++ show parseError
+    show (IOError {..}) = intercalate ", " (map showLoc locs) ++ ": IOError " ++ show ioError
+
+showLoc :: Loc -> String
+showLoc x = show (loc_filename x) ++ "(line " ++ show (fst (loc_start x)) ++ ", column " ++ show (snd (loc_start x)) ++ ")"
 
 -- instance Please.Show ControlFileError where
 --     show (IOError e) = "(IOError " ++ Please.show e ++ ")"
@@ -115,7 +128,7 @@ instance Eq ControlFileError where
 debianPackageParagraphs :: HasDebianControl a => a -> (Paragraph' Text, [Paragraph' Text])
 debianPackageParagraphs ctl =
     case removeCommentParagraphs ctl of
-      DebianControl (Control [_]) -> throw $ NoBinaryParagraphs [$__LOC__]
+      DebianControl (Control [_]) -> throw $ NoBinaryParagraphs [$__LOC__] (show ctl)
       DebianControl (Control []) -> throw $ NoParagraphs [$__LOC__]
       DebianControl (Control (sourceParagraph : binParagraphs)) -> (sourceParagraph, binParagraphs)
 
