@@ -1,15 +1,22 @@
+-- | A constructor we can wrap around values to avoid any built in
+-- Pretty instance - for example, instance Pretty [a].
+--
+--  * display is now prettyShow
+--  * display' is now prettyText
+--  * ppDisplay is now ppShow
+--  * ppDisplay' is now ppText
 {-# LANGUAGE DeriveFunctor, FlexibleContexts, FlexibleInstances, OverloadedStrings, TypeSynonymInstances #-}
 module Debian.Pretty
     ( PP(PP, unPP)
-    , display
-    , display'
+    , prettyShow
+    , prettyText
     , ppPrint
-    , ppDisplay
-    , ppDisplay'
+    , ppShow
+    , ppText
     ) where
 
 import Data.Text (Text, unpack, pack)
-import Text.PrettyPrint.HughesPJClass (Doc, Pretty(pPrint), text, empty)
+import Text.PrettyPrint.HughesPJClass (Doc, Pretty(pPrint), text, empty, prettyShow)
 
 -- | This type is wrapped around values before we pretty print them so
 -- we can write our own Pretty instances for common types without
@@ -26,17 +33,14 @@ instance Pretty (PP String) where
 instance Pretty (PP a) => Pretty (PP (Maybe a)) where
     pPrint = maybe empty ppPrint . unPP
 
-display :: Pretty a => a -> String
-display = show . pPrint
-
-display' :: Pretty a => a -> Text
-display' = pack .show . pPrint
+prettyText :: Pretty a => a -> Text
+prettyText = pack . prettyShow
 
 ppPrint :: Pretty (PP a) => a -> Doc
 ppPrint = pPrint . PP
 
-ppDisplay :: Pretty (PP a) => a -> String
-ppDisplay = display . PP
+ppShow :: Pretty (PP a) => a -> String
+ppShow = prettyShow . PP
 
-ppDisplay' :: Pretty (PP a) => a -> Text
-ppDisplay' = pack . display . PP
+ppText :: Pretty (PP a) => a -> Text
+ppText = pack . prettyShow . PP
